<?php
/**
 * DiffEngine implementuje metody pro hledání rozdílů mezi polema uživatelů User
 * a místností Room.
 *
 * @author xharazim
 */
class DiffEngine implements IDiffEngine, ILogable {

    private $className = 'DiffEngine';

    /**
     * Vrátí uživatele, kteří jsou v incomingUsers a nejsou v currentUsers, tzn.
     * nově příchozí uživatele určené pro další zpracování.
     * @param <type> $currentUsers pole současných uživatelů
     * @param <type> $incomingUsers pole příchozích uživatelů
     * @return pole uživatelů
     */
    public function findNewUsers($currentUsers, $incomingUsers) {

        $methodName = 'findNewUsers';
        $this->logDebug('Entering method '.$methodName, $methodName);

        //expect two arrays of User, result is array of new users
        $result = array();
        $result = $this->subtract($incomingUsers,$currentUsers);
        return $result;
    }

        /**
     * Vrátí uživatele, kteří nejsou v incomingUsers a jsou v currentUsers, tzn.
     * nově chybějící uživatele určené pro další zpracování.
     * @param <type> $currentUsers pole současných uživatelů
     * @param <type> $incomingUsers pole příchozích uživatelů
     * @return pole uživatelů
     */
    public function findUsersToDelete($currentUsers, $incomingUsers) {

                $methodName = 'findUsersToDelete';
        $this->logDebug('Entering method '.$methodName, $methodName);

        //expect two arrays of User, result is array of users to delete
        $result = array();
        $result = $this->subtract($currentUsers, $incomingUsers);
        return $result;
    }

    /**
     * Vrátí místnosti, které jsou v incomingRooms a nejsou v currentRooms, tzn.
     * nově příchozí místnosti.
     * @param <type> $currentRooms pole současných místnosti
     * @param <type> $incomingRooms pole příchozích místnosti
     * @return pole místností
     */
    public function findNewRooms($currentRooms, $incomingRooms) {

                $methodName = 'findNewRooms';
        $this->logDebug('Entering method '.$methodName, $methodName);

        //return array_diff($currentRooms, $incomingRooms);
        $result = array();
        $result = $this->subtractRooms($incomingRooms, $currentRooms);
        return $result;
    }

        /**
     * Vrátí místnosti, které nejsou v incomingRooms a jsou v currentRooms, tzn.
     * místnoti chybějící v nově příchozích místnostech.
     * @param <type> $currentRooms pole současných místnosti
     * @param <type> $incomingRooms pole příchozích místnosti
     * @return pole místností
     */
    public function findRoomsToDelete($currentRooms, $incomingRooms) {

                $methodName = 'findRoomsToDelete';
        $this->logDebug('Entering method '.$methodName, $methodName);

        //return array_diff($incomingRooms, $currentRooms);
        $result = array();
        $result = $this->subtractRooms($currentRooms, $incomingRooms);
        return $result;
    }

    // private functions -----------------------------------------------------

    /*
     * This function makes subtract on array1 and array2 in this way:
     * subtract(array1, array2) == subtract of array1 and array2, array1-array2
     *
         * @param <type> $array1 pole instancí User
         * @param <type> $array2 pole instancí User
         * @return pole instancí User
    */
    private function subtract($array1, $array2) {

        $methodName = 'subtract';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $result = array();
        $index = 0;
        $isInIncoming = false; // expect the user is not in new array..
        foreach ($array1 as $user1) {
            $eppn = $user1->getEppn(); // check equal user by eppn
            foreach($array2 as $user2) {
                if ($eppn == $user2->getEppn()) {
                    $isInIncoming = true;
                    break;
                }
            }
            if (!$isInIncoming) {
                // delete curUser
                $result[$index] = $user1;
                $index++;

        $this->logDebug('Diff user found:  ['.$user1->toString().']', $methodName);
            }
            $isInIncoming = false;
        }

        $this->logDebug('subtract result count: ['.count($result).']', $methodName);
        return $result;
    }

        /**
         * Vrátí rozdíl místností: result = array1 - array2
         * @param <type> $array1
         * @param <type> $array2
         * @return <type>
         */
    private function subtractRooms($array1, $array2) {

        $methodName = 'subtractRooms';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $result = array();
        $index = 0;
        $isInArray2 = false; // expect the room is not in array2..
        foreach ($array1 as $room1) {
            $name = $room1->getName(); // check equal room by name, unique in AC
            foreach($array2 as $room2) {
                if ($name == $room2->getName()) {
                    $isInArray2 = true;
                    break;
                }
            }
            if (!$isInArray2) {
  $this->logDebug('Diff room found:  ['.$room1->toString().']', $methodName);
                $result[$index] = $room1;
                $index++;
            }
        }

        $this->logDebug('subtractRooms result count: ['.count($result).']', $methodName);
        return $result;
    }

    public function logDebug($description, $methodName) {
        SysLogger::logDebug($description, $this->className, $methodName);
    }

    public function logError($description, $methodName) {
        SysLogger::logError($description, $this->className, $methodName);
    }
}
?>
