#!/bin/bash

PROTOCOL_VERSION='3.0.0'


function process {
  ### Status codes
  I_K5LOGIN_CREATED=(0 '${HOME_DIR}/.k5login with entries ${PRINCIPALS} created.')
  I_K5LOGIN_UPDATED=(0 '${HOME_DIR}/.k5login updated. Added entries are ${ADDED_PRINCIPALS}.')

  E_DIR_NOT_EXISTS=(50 'Home directory ${HOME_DIR} does not exits.')

  FROM_PERUN="${WORK_DIR}/k5login"

  create_lock

  # lines contains login\tprincipal#1\tprincipal#2\t...
  while read line
  do
    HOME_DIR=`echo "${line}" | awk '{ print $1 };'`
    PRINCIPALS=`echo "${line}" | sed -e 's/^[^\t]*[\t]//'`
    
    catch_error E_DIR_NOT_EXISTS cd "${HOME_DIR}"
    
    # If .k5login doesn't exists, create it and fill it with proper principals
    if [ ! -f "${HOME_DIR}/.k5login" ]; then
      for principal in ${PRINCIPALS}; do
        echo "${principal}" >> "${HOME_DIR}/.k5login"
      done

      # Setup rights, the .k5login will have the same owner and group as user's home directory
      F_USER=`ls -ld "${HOME_DIR}" | awk '{ print $3; }'`
      F_GROUP=`ls -ld "${HOME_DIR}" | awk '{ print $4; }'`
      chown ${F_USER}.${F_GROUP} "${HOME_DIR}/.k5login"
      chmod 0644 "${HOME_DIR}/.k5login"

      log_msg I_K5LOGIN_CREATED
    else
      # .k5login exists. So only add missing entries.
      ADDED_PRINCIPALS=""
      for principal in ${PRINCIPALS}; do
        grep "^${principal}\\s*\$" "${HOME_DIR}/.k5login" > /dev/null
        if [ $? -eq 1 ]; then 
          ADDED_PRINCIPALS="$ADDED_PRINCIPALS ${principal}"
          echo "${principal}" >> "${HOME_DIR}/.k5login"
        fi
      done

      if [ ! -z "$ADDED_PRINCIPALS" ]; then 
        log_msg I_K5LOGIN_UPDATED
      fi
    fi
  done < "${FROM_PERUN}"
}
